BACnet.Protocol.APDU.SegmentACK (bacstack v0.0.1)
View SourceSegment ACK APDUs are used to acknowledge the receipt of one or more frames containing portions of a segmented message. It may also request the next segment or segments of the segmented message.
This module has functions for encoding Segment ACK APDUs.
Decoding is handled by BACnet.Protocol.APDU.
This module implements the BACnet.Stack.EncoderProtocol.
Summary
Functions
Encodes the Segment ACK APDU into binary data.
Types
Functions
@spec encode(t()) :: {:ok, iodata()} | {:error, Exception.t()}
Encodes the Segment ACK APDU into binary data.